/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.ant.taskdefs;

import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

import javassist.NotFoundException;
import javassist.gluonj.Gluon;
import javassist.gluonj.WeaveException;
import javassist.gluonj.weave.OfflineWeaver;

/**
 * Ant task for compile-time weaving.
 */
public class Weave extends Task {
    private Path classpath;
    private String glueName;
    private ArrayList filesets;
    private String destdir;

    public Weave() {
        classpath = null;
        glueName = null;
        filesets = new ArrayList();
        destdir = ".";
    }

    public void setClasspath(Path path) {
        if (classpath == null)
                classpath = new Path(getProject());

        classpath.append(path);
    }

    public void setClasspathRef(Reference ref) {
        createClasspath().setRefid(ref);
    }

    public Path createClasspath() {
        if (classpath == null)
            classpath = new Path(getProject());

        return classpath.createPath();
    }

    public String[] getClasspath() {
        if (classpath != null)
            return classpath.list();

        return new String[0];
    }

    public void setGlue(String className) {
        glueName = className;
    }

    public void setDestdir(String dir) {
        destdir = dir;
    }

    public void setDebug(String value) {
        Gluon.stackTrace
            = value.equals("yes") || value.equals("true")
              || value.equals("on");
    }

    public String getDestdir() {
        return destdir;
    }

    public void addFileset(FileSet fileset) {
        filesets.add(fileset);
    }

    /**
     * Returns a List<FileSet> object. 
     */
    public List getFileSets() { return filesets; }

    public void execute() throws BuildException {
        Exception ex = null;
        try {
            execute0();
            return;
        }
        catch (NotFoundException e) { ex = e; }
        catch (WeaveException e) { ex = e; }
        catch (RuntimeException e) { ex = e; }
        if (Gluon.stackTrace)
            ex.printStackTrace();

        throw new BuildException(ex.toString(), ex);
    }

    private void execute0() throws BuildException, WeaveException, NotFoundException {
        if (glueName == null)
            throw new BuildException("a glue name is not specified");

        OfflineWeaver weaver = new OfflineWeaver(glueName, getClasspath());
        if (destdir != null)
            weaver.setDirectory(destdir);

        Iterator it = filesets.iterator();
        while (it.hasNext()) {
            FileSet fs = (FileSet)it.next();
            DirectoryScanner ds
                = fs.getDirectoryScanner(getProject());
            String[] files = ds.getIncludedFiles();
            weaver.transformFiles(ds.getBasedir(), files);
        }

        weaver.done();
    }
}
