/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.pc;

import javassist.CtBehavior;
import javassist.CannotCompileException;
import javassist.gluonj.WeaveException;
import javassist.gluonj.runtime.Cflow;

public class CflowPc extends AccessPc {
    private static String threadVar = Cflow.class.getName() + ".singleton"; 
    private static int uniqueNo = 0;
    private String cflowName;
    private int counter;

    public CflowPc(String pat) {
        super(pat);
        cflowName = "_" + uniqueNo++;
        counter = 0;
    }

    public String getExpr() throws RuntimeException {
        return threadVar + ".check(\"" + cflowName + "\")";
    }

    public void accept(PointcutVisitor v) throws WeaveException {
        v.visit(this);
    }

    public String toString() {
        return "cflow(" + patternStr + ")";
    }

    public void prepare(PatternParser p) throws WeaveException {
        super.prepare(p);
        if (!pattern.isMethod())
            throw new WeaveException("bad pattern: " + toString());

        cflowName = p.getGlueName() + cflowName;
    }

    public void instrument(CtBehavior method)
        throws WeaveException
    {
        String className = method.getDeclaringClass().getName();
        if (getPattern().matchMethod(className, method.getName(),
                                     method.getSignature()))
            try {
                if (counter++ > 0)
                    throw new WeaveException("cflow() cannot include an wild card");

                method.insertBefore(threadVar + ".enter(\"" + cflowName + "\");");
                method.insertAfter(threadVar + ".exit(\"" + cflowName + "\");", true);
            }
            catch (CannotCompileException e) {
                throw new WeaveException(e);
            }
    }
}
