/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
 package javassist.gluonj.util;

import javassist.CtClass;
import javassist.expr.FieldAccess;
import javassist.expr.MethodCall;
import javassist.expr.NewExpr;
import javassist.gluonj.WeaveException;
import javassist.gluonj.pc.PatternParser;
import javassist.gluonj.pc.PointcutNode;
import javassist.gluonj.pc.PointcutVisitor;
import javassist.gluonj.weave.Inspector;
import javassist.gluonj.weave.Residue;

/**
 * A utility class for implementing a new kind of pointcut node.
 * 
 * @author Muga Nishizawa
 * @author Shigeru Chiba
 */
public class SimplePcNode extends PointcutNode implements Inspector {
    /**
     * Accepts a visitor.
     */
    public void accept(PointcutVisitor v) throws WeaveException {
        v.visit(this);
    }

    /**
     * Is called just after a pointcut expression is parsed.  It prepares
     * for joinpoint matching.
     */
    public void prepare(PatternParser p) throws WeaveException {}

    /**
     * Is called just before an advice weaver starts processing
     * each class definition.  The <code>prepare</code> method is called
     * before any call to this method.  This method can perform static
     * analysis of the definition of the given class.  Changing the class
     * definition is not recommended.
     * 
     * <p><code>Pointcut.PrepareVisitor#visit(SimplePcNode)</code> registers
     * this object as an inspector.
     */
    public void inspect(CtClass clazz) throws WeaveException {
        // if the name of this method is changed, edit overriddenInspect()
        // in Pointcut.PrepareVisitor.
    }

    /**
     * Returns true if the given join point matches the pointcut.
     * If it matches and there is a join point residue, the residue
     * must be stored in <code>residue[0]</code>.
     *
     * @param residue       a single-element array where a return
     *                      value will be stored.
     */
    public boolean match(MethodCall mc, Residue[] residue) throws WeaveException {
        residue[0] = null;
        return false;
    }

    /**
     * Returns true if the given join point matches the pointcut.
     * If it matches and there is a join point residue, the residue
     * must be stored in <code>residue[0]</code>.
     *
     * @param residue       a single-element array where a return
     *                      value will be stored.
     */
    public boolean match(FieldAccess fa, Residue[] residue) throws WeaveException {
        residue[0] = null;
        return false;
    }

    /**
     * Returns true if the given join point matches the pointcut.
     * If it matches and there is a join point residue, the residue
     * must be stored in <code>residue[0]</code>.
     *
     * @param residue       a single-element array where a return
     *                      value will be stored.
     */
    public boolean match(NewExpr ne, Residue[] residue) throws WeaveException {
        residue[0] = null;
        return false;
    }
}