package javassist.gluonj.weave;

import javassist.expr.FieldAccess;
import javassist.gluonj.WeaveException;
import javassist.gluonj.pc.GetPc;
import javassist.gluonj.pc.SetPc;
import javassist.gluonj.util.SimplePcNode;

/**
 * A Matcher customized for field-access join points.
 */
public class FieldMatcher extends ExprMatcher {
    private FieldAccess joinPoint;
    private String declClass;
    private String fieldName;
    private boolean isRead;

    public FieldMatcher(FieldAccess fa) {
        super(fa.where());
        joinPoint = fa;
        declClass = fa.getClassName();
        fieldName = fa.getFieldName();
        isRead = fa.isReader();
    }

    public void visit(SetPc pc) throws WeaveException {
        if (isRead)
            result = false;
        else
            result = pc.getPattern().matchField(declClass, fieldName);
    }

    public void visit(GetPc pc) throws WeaveException {
        if (isRead)
            result = pc.getPattern().matchField(declClass, fieldName);
        else
            result = false;
    }

    public void visit(SimplePcNode pc) throws WeaveException {
        Residue[] r = new Residue[1];
        result = pc.match(joinPoint, r);
        residue = r[0];
    }
}

